// $Header: E:/TNG/src/rcs/gtype.cpp 1.2 1997/02/22 14:37:50 dbm Exp $

#include <stdlib.h> // for rand()
#include "gtype.h"    
#include "random.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	bioGType
//

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	bioHapGType
//   

bioHapGType::bioHapGType (void)
{
	_bits = 0;
	_bytes = 0;
	_dna = 0;
}

bioHapGType::bioHapGType (bioHapGType& parent)
{
	_bits = 0;
	_bytes = 0;
	_dna = 0;

	(*this) = parent;
}

bioHapGType::~bioHapGType (void)
{
	delete _dna;
}
 
bioHapGType&
bioHapGType::operator= (const bioHapGType& rhs)
{        
	if (this == &rhs)
	{
		return *this;
	}
	
	if (_bytes != rhs._bytes)
	{
		if (_dna != 0)
		{
			delete _dna;
		}               

		_bits = rhs._bits;		                        
		_bytes = rhs._bytes;		                        
		_dna = new char[_bytes];  
		ASSERT (_dna);
	}

	for (int i = 0; i < _bytes; i++)
	{
		_dna[i] = rhs._dna[i];
	}
	
	return *this;
}


void
bioHapGType::SetLength (int length)
{
	assert (length > 0);
	assert (_dna == 0);
	
	_bits = length;
	_bytes = (_bits-1)/8 + 1;
	_dna = new char[_bytes];
	ASSERT (_dna);
}

int
bioHapGType::GetLength (void)
{
	return _bits;
}

#include <stdio.h>

void
bioHapGType::Randomize (void)
{             
	assert (_dna != 0);

	/*
	for (int i = 0; i < _bytes; i++)
	{
		_dna[i] = rand();
	}*/
	for (int i = 0; i < _bits; i++)
	{
		int			x = rand();

		SetBit(i, (x & 1) == 1);
		//printf("randomize %d %d %d\n", i, x, (x&1));
	}
}

char
bioHapGType::GetByte (int index)
{             
	assert (index >= 0);
	assert (index < _bytes);
	
	return _dna[index];
}

int
bioHapGType::GetBit (int index)
{         
	assert (index >= 0);
	assert (index < _bits);
  	
	char		byte = _dna[index/8];
	int			bit = 7 - (index%8);

	return ((byte & (1<<bit)) > 0);
}

void
bioHapGType::SetBit (int index, int on)
{
	assert (index >= 0);
	assert (index < _bits);
  		
	// Changed to be consistent with Java version
	// dbm 97.04.06.97
	char			mask = (1 << (7 - (index%8)));

	if (on)
	{
		_dna[index/8] |= mask;
	}
	else
	{
		_dna[index/8] &= ~mask;
	}
}

void
bioHapGType::ResetBit (int index)
{
	SetBit (index, 0);
}

void
bioHapGType::ToggleBit (int index)
{   
	assert (index >= 0);
	assert (index < _bits);
  	
	SetBit (index, !GetBit(index));
}

void
bioHapGType::Mutate (double mrate)
{
	unsigned int	next;      	
	double			denom = log(1.0 - mrate);

	next = (int) ceil (log(DRand())/denom);                     
    
	while (next < (unsigned int)_bits)
	{                                  
		ToggleBit(next); 
		//printf ("mutate bit #%d\n", next);
		next += (int) ceil (log(DRand())/denom);              
	}
}

bioGType*
bioHapGType::Crossover (bioGType* other)
{
	bioHapGType&	parent = *(bioHapGType*) other; 
	bioHapGType*	kid = new bioHapGType (parent);       
	
	ASSERT (kid);
	
	int				x1 = rand() % _bits;
	int				x2 = rand() % (_bits-1);
	int				i;

	if (x2 >= x1) 
	{
		x2++;
	}
	else
	{
		i = x1;
		x1 = x2;
		x2 = i;
	}
     
	for (i = x1; i < x2; i++)
	{
		kid->SetBit (i, GetBit(i));
	}
	
	//printf ("crossover %d to %d\n", x1, x2);

	return kid;
}
  

