// $Header: E:/TNG/src/rcs/list.cpp 1.2 1997/02/22 14:38:04 dbm Exp $

#include <stdio.h>
#include <iostream.h>
#include "list.h"

bioNode::bioNode (bioObject* obj)
{
	data = obj;
	prev = 0;
	next = 0;
}

bioList::bioList () 
{                          
	_count = 0;
 	_first = 0;
	_cur = 0;
	_last = 0;
}

bioList::bioList (bioList& rhs) 
{                          
	_count = 0;
 	_first = 0;
	_cur = 0;
	_last = 0;

	(*this) = rhs;  
}

bioList::~bioList() 
{
	ASSERT_VALID (this);
	
	bioNode*		node;
	
  	while (_last != 0) 
  	{
    	node = _last;
    	_last = node->prev;
    	delete node;
  	};
}

bioList&
bioList::operator = (bioList& rhs)
{
	ASSERT_VALID (&rhs);
	                    
	bioNode*		node;
	bioObject*		obj;
	
  	while (_last != 0) 
  	{
    	node = _last;
    	_last = node->prev;
    	delete node;
  	};
  	
  	for (rhs.First(); obj = rhs.Next(); )
  	{
  		Append (obj);
  	}                
	
	ASSERT_VALID (this);
	
	return *this;
}

int
bioList::Count (void)
{
	return _count;
}

int
bioList::IsEmpty (void)
{
	return (_count == 0);
}

int 
bioList::Append (bioObject* obj) 
{
  	bioNode*	node = new bioNode (obj);
  
  	ASSERT (node);
  	
  	if (node == 0) 
  	{
    	return FALSE;
  	};
	
	if (_first == 0)
	{
		_first = _cur = _last = node;
	}
	else
	{
		if (_cur == 0)
		{
			_cur = _last;
		}
		
		if (_cur == _last)
		{
			_last = node;
		}
		   
		if (_cur->next)
		{
			_cur->next->prev = node;
			node->next = _cur->next;
		}
		
		_cur->next = node;
		node->prev = _cur;
		
		_cur = node;
	}       
	
	++_count;
	
	return TRUE;
}
               
void
bioList::First (void)
{
	_cur = _first;
}

bioObject*
bioList::Next (void)
{
	bioObject*		obj = 0;
	
	if (_cur)
	{
		obj = _cur->data;            
		_cur = _cur->next;
	}

	return obj;
}
               
void
bioList::Last (void)
{
	_cur = _last;
}

bioObject*
bioList::Prev (void)
{
	bioObject*		obj = 0;
	
	if (_cur)
	{
		obj = _cur->data;            
		_cur = _cur->prev;
	}

	return obj;
}

int
bioList::Remove (bioObject* obj)
{   
	ASSERT_VALID (obj);
	
	bioNode*		node = _Find (obj);
	
	if (node)
	{    
		if (node == _first)
		{
			_first = _first->next;
		}
		if (node == _last)
		{
			_last = _last->prev;
		}
		if (node == _cur)
		{
			_cur = 0;
		}            
		
		if (node->next)
		{
			node->next->prev = node->prev;
		} 
		
		if (node->prev)
		{
			node->prev->next = node->next;
		}
		
		delete node;
		--_count; 
		
		return TRUE;
	}               
	else
	{
		return FALSE;
	}
}

int
bioList::DeleteAll (void)
{
	bioObject*		data;
	
	while (_count > 0)
	{
		data = _first->data;
		Remove (_first->data);
		delete data;
	}                         
	
	return TRUE;
}

int
bioList::RemoveAll (void)
{
	while (_count > 0)
	{
		Remove (_first->data);
	}                         
	
	return TRUE;
}

int
bioList::Find (bioObject* obj)
{   
	ASSERT_VALID (obj);
	
	_cur = _Find (obj);
	
	return (_cur == 0) ? FALSE : TRUE;
}

bioNode*
bioList::_Find (bioObject* obj)
{
	bioNode*		node = _first;
	
	while (node)
	{
		if (node->data == obj)
		{       
			return node;
		}               
		
		node = node->next;
	}                     
	
	return 0;
}       

void
bioList::AssertValid (void) const
{
	bioObject::AssertValid();
	
	if (_first == 0)
	{                       
		ASSERT (_count == 0);
		ASSERT (_cur == 0);
		ASSERT (_last == 0); 
		return;
	}                      

	ASSERT (_first->prev == 0);
	ASSERT (_last);
	ASSERT (_last->next == 0);

	bioNode*		node = _first; 
	int				count = 0;
		           
	while (node)
	{                         
		++count;
		
		if (node->prev)
		{
			ASSERT (node->prev->next == node);
		}
		
		if (node->next)
		{
			if (node->next->prev != node)
			{                     
				count = -1;
			}
			
			ASSERT (node->next->prev == node);
		}
		
		ASSERT_VALID (node->data);

		node = node->next;
	}           
	
	ASSERT (_count == count);
}

#ifdef LIST_TEST

void 
main() 
{
	int 			i;
	bioList*		list;
	
	list = new bioList;

	ASSERT_VALID(list);
	
	cout << "Building bioList\n";
	
	for (i = 0; i < 10; i++) 
	{
		list->Append (new bioObject);
		
		ASSERT_VALID(list);
	
		cout << i << " ";
	}
	cout << endl;
	     
    bioObject* obj;               
	
	cout << "forwards" << endl;
	
	for (list->First(); obj = list->Next(); )
	{
		ASSERT_VALID(list);
		cout << obj << endl;
	}
	   
	bioList			other = *list;
	
	ASSERT_VALID (&other);
	
	cout << "count " << other.Count() << endl;
	cout << "backwards" << endl;

	for (list->Last(); obj = list->Prev(); )
	{   
		ASSERT_VALID(list);
		cout << obj << endl;
		list->Remove (obj);
	}
	                    
	cout << "count " << list->Count() << endl; 
	ASSERT_VALID(list);
  	delete list;
}

#endif // LIST_TEST
