// $Header: E:/TNG/src/rcs/world.cpp 1.2 1997/02/22 14:38:20 dbm Exp $

#include "world.h"
#include "xmatrix.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	bioWorld
//
 
bioWorld::bioWorld (void)
{
}
     
bioWorld::~bioWorld (void)
{
}

void
bioWorld::_SetPosition (bioThing* thing, double x, double y)
{ 
	ASSERT_VALID (thing);
	thing->__x = x;
	thing->__y = y;
}
    
void     
bioWorld::AddThing (bioThing* thing, double x, double y)
{
	ASSERT_VALID (thing);
	                                  
	_things.Append (thing);
	_SetPosition (thing, x, y);
	thing->_world = this;

	Trace ("AddThing(%p)\n", thing);
}     
           
void           
bioWorld::RemoveThing (bioThing* thing)
{                                  
	ASSERT_VALID (thing);
	
	ASSERT (_things.Find (thing) == TRUE);
	
	_things.Remove (thing);
	thing->_world = 0;

	ASSERT (_things.Find (thing) == FALSE); 
	
	Trace ("RemoveThing(%p)\n", thing);
} 

int
bioWorld::ThingCount (void)
{
	return _things.Count();
}

void
bioWorld::MoveThing (bioThing* thing, double x, double y, int relative)
{
	ASSERT_VALID (thing);
	RemoveThing (thing);
	
	if (relative)
	{ 
		AddThing (thing, thing->__x + x, thing->__y + y);
	}
	else
	{
		AddThing (thing, x, y);
	}
}
   
void
bioWorld::TurnThing (bioThing* thing, double angle, int relative)
{
	ASSERT_VALID (thing);

	if (relative)
	{ 
		thing->__orientation += angle;
	}
	else
	{
		thing->__orientation = angle;
	}
	
	while (thing->__orientation < 0)
	{
		thing->__orientation += 360;
	}
	
	while (thing->__orientation >= 360)
	{
		thing->__orientation -= 360;
	}
}

void
bioWorld::XformPos (double x, double y, bioThing* thing, double& nx, double& ny)
{ 
	XMatrix			mat;
	Point			focus (Round (thing->__x + x), Round (thing->__y - y));
	Point			pos;
	
	// The __orientation angle is in degrees so we have to convert it to radians.
	mat.RotateAboutPoint (DEGREES(thing->__orientation), thing->__x, thing->__y);
	pos = focus * mat;
	
	nx = pos.x;
 	ny = pos.y;
}   

void
bioWorld::NextStep (void)
{
	//DWORD				ticks = timeGetTime();
	bioList				list;
	bioThing*			thing ;
		
	// Copy all things to local list.
	list = _things;
	
    // Tell all things to have a look.
	for (list.First(); thing = (bioThing*) list.Next(); )
	{
		ASSERT_VALID (thing);
		thing->Look();
	}
                                      
	// Resolve all things' intentions.                                      
	for (list.First(); thing = (bioThing*) list.Next(); )
	{
		ASSERT_VALID (thing);
		ResolveAction (thing, thing->GetIntent());
	} 
	                                  
	// Kill the things on deathrow.	                                  
	_ExecuteThings(); 
}
             
void
bioWorld::DeleteThing (bioThing* thing)
{                                     
	ASSERT_VALID (thing);
	
	RemoveThing (thing);
	_deathRow.Append (thing);
	Trace ("--- added %p to death row\n", thing);
}

void
bioWorld::_ExecuteThings (void)
{
	ASSERT_VALID (&_deathRow);
	
	_deathRow.DeleteAll();
}

